@@ -5,12 +5,14 @@ import Language.Haskell.HsColour
 import qualified Language.Haskell.HsColour as HSColour
 import Language.Haskell.HsColour.Colourise (readColourPrefs)
 import Language.Haskell.HsColour.Options
-import System
-import IO
-import System.IO (withFile)
-import Monad (when)
-import List  (intersperse, isSuffixOf)
-import Debug.Trace
+import Language.Haskell.HsColour.ACSS (breakS, srcModuleName)
+import System.Environment as System
+import System.Exit
+import System.IO hiding (withFile)
+import Control.Monad (when, forM_)
+import Data.List  (intersperse, isSuffixOf)
+import Control.Exception (bracket)
+--import Debug.Trace
 
 -- Deal with UTF-8 I/O.
 #if __GLASGOW_HASKELL__ > 611
@@ -18,7 +20,7 @@ import Debug.Trace
 import System.IO (hSetEncoding, utf8)
 #endif
 
-version = "1.16"
+version = "1.19"
 
 optionTable :: [(String,Option)]
 optionTable = [ ("help",    Help)
@@ -28,6 +30,7 @@ optionTable = [ ("help",    Help)
               , ("css",    Format CSS)
               , ("icss",   Format ICSS)
               , ("tty",    Format TTY)
+              , ("tty256", Format (TTYg XTerm256Compatible))
               , ("latex",  Format LaTeX)
               , ("mirc",   Format MIRC)
               , ("lit",    LHS True)
@@ -41,6 +44,7 @@ optionTable = [ ("help",    Help)
 
 parseOption :: String -> Either String Option
 parseOption ('-':'o':s) = Right (Output s)
+parseOption ('-':'a':'n':'n':'o':'t':'=':s) = Right (Annot s)
 parseOption s@('-':_)   = maybe (Left s) Right
                                 (lookup (dropWhile (== '-') s) optionTable)
 parseOption s           = Right (Input s)
@@ -55,14 +59,15 @@ main = do
       good    = [ o | Right o <- options ]
       formats = [ f | Format f <- good ]
       outFile = [ f | Output f <- good ]
+      annFile = [ f | Annot f <- good ]
       output    = useDefault  TTY         id           formats
       anchors   = useDefault  False       id           [ b | Anchors b <- good ]
       partial   = useDefault  False       id           [ b | Partial b <- good ]
       lhs       = useDefault  Nothing     id           [ Just b | LHS b<- good ]
       title     = useDefault  "Haskell code" id        [ f | Input f   <- good ]
       ioWrapper = useDefaults (ttyInteract  outFile (guessLiterate lhs ""))
-                              (fileInteract outFile)   [ (f,guessLiterate lhs f)
-                                                           | Input f   <- good ]
+                              (fileInteract outFile annFile)   
+			      [(f, guessLiterate lhs f) | Input f <- good]
   when (not (null bad)) $
        errorOut ("Unrecognised option(s): "++unwords bad++"\n"++usage prog)
   when (Help `elem` good)        $ writeResult [] (usage prog)
@@ -73,20 +78,24 @@ main = do
                  ++unwords (map show formats))
   when (length outFile > 1) $
        errorOut ("Can only have one output file at a time.")
+  when (length annFile > 1) $
+       errorOut ("Can only use a single annotation file for annotated-CSS output")
+  
   ioWrapper (HSColour.hscolour output pref anchors partial title)
 
   where
     writeResult outF s = do if null outF then putStr s
                                          else writeUTF8File (last outF) s
                             exitSuccess
-    fileInteract out inFs u = do h <- case out of
-                                          []     -> return stdout
-                                          [outF] -> openFile outF WriteMode >>=
-                                                    set_utf8_io_enc
-                                 mapM_ (\ (f,lit)->
-                                           readUTF8File f >>= hPutStr h . u lit)
-                                       inFs
-                                 hClose h
+    fileInteract out ann inFs u 
+      = do h <- case out of
+                  []     -> return stdout
+                  [outF] -> openFile outF WriteMode >>= set_utf8_io_enc
+           forM_ inFs $ \ (f, lit) -> do
+             src <- readUTF8File f
+             a   <- readAnnots src ann
+             hPutStr h $ u lit $ src ++ a 
+           hClose h
     ttyInteract []     lit u = do hSetBuffering stdout NoBuffering
                                   Prelude.interact (u lit)
     ttyInteract [outF] lit u = do c <- hGetContents stdin
@@ -97,6 +106,7 @@ main = do
                   ++" options [file.hs]\n    where\n      options = [ "
                   ++ (indent 15 . unwords . width 58 58 . intersperse "|"
                      . ("-oOUTPUT":)
+                     . ("-annot=ANNOTATIONFILE":)
                      . map (('-':) . fst)) optionTable
                   ++ " ]\n"
     useDefault d f list  | null list = d
@@ -106,6 +116,12 @@ main = do
     guessLiterate Nothing  f = ".lhs" `isSuffixOf` f || ".ly" `isSuffixOf` f
                                || ".lx" `isSuffixOf` f
     guessLiterate (Just b) _ = b
+    readAnnots _   []     = return ""
+    readAnnots src [annF] = do putStrLn $ "HsColour Annot on Module: " ++ mname
+                               annots <- readUTF8File annF
+                               return $ breakS ++ "\n" ++ mname ++ "\n" ++ annots
+                            where mname = srcModuleName src
+
 
 -- some simple text formatting for usage messages
 width n left  []    = []
@@ -156,3 +172,7 @@ writeUTF8File f txt = withFile f WriteMode (\hdl -> do set_utf8_io_enc hdl
 readUTF8File name   = openFile name ReadMode >>=
                       set_utf8_io_enc >>=
                       hGetContents
+
+withFile :: FilePath -> IOMode -> (Handle -> IO r) -> IO r
+withFile name mode = bracket (openFile name mode) hClose
+
