@@ -11,6 +11,7 @@ import Language.Haskell.HsColour.Classify as Classify
 import Language.Haskell.HsColour.Colourise
 
 import Data.Char(isAlphaNum)
+import Text.Printf
 
 
 -- | Formats Haskell source code using HTML with font tags.
@@ -36,8 +37,7 @@ renderToken :: ColourPrefs -> (TokenType,String) -> String
 renderToken pref (t,s) = fontify (colourise pref t)
                          (if t == Comment then renderComment s else escape s)
 
-renderAnchors :: ((TokenType,String)->String)
-                 -> Either String (TokenType,String) -> String
+renderAnchors :: (a -> String) -> Either String a -> String
 renderAnchors _      (Left v) = "<a name=\""++v++"\"></a>"
 renderAnchors render (Right r) = render r
 
@@ -60,9 +60,11 @@ renderNewLinesAnchors = unlines . map render . zip [1..] . lines
     where render (line, s) = "<a name=\"line-" ++ show line ++ "\"></a>" ++ s
 
 -- Html stuff
+fontify ::  [Highlight] -> String -> String
 fontify [] s     = s
 fontify (h:hs) s = font h (fontify hs s)
 
+font ::  Highlight -> String -> String
 font Normal         s = s
 font Bold           s = "<b>"++s++"</b>"
 font Dim            s = "<em>"++s++"</em>"
@@ -70,23 +72,28 @@ font Underscore     s = "<u>"++s++"</u>"
 font Blink          s = "<blink>"++s++"</blink>"
 font ReverseVideo   s = s
 font Concealed      s = s
-font (Foreground c) s = "<font color="++show c++">"++s++"</font>"
+font (Foreground (Rgb r g b)) s = printf   "<font color=\"#%02x%02x%02x\">%s</font>" r g b s
+font (Background (Rgb r g b)) s = printf "<font bgcolor=\"#%02x%02x%02x\">%s</font>" r g b s
+font (Foreground c) s =   "<font color="++show c++">"++s++"</font>"
 font (Background c) s = "<font bgcolor="++show c++">"++s++"</font>"
 font Italic         s = "<i>"++s++"</i>"
 
+escape ::  String -> String
 escape ('<':cs) = "&lt;"++escape cs
 escape ('>':cs) = "&gt;"++escape cs
 escape ('&':cs) = "&amp;"++escape cs
 escape (c:cs)   = c: escape cs
 escape []       = []
 
+htmlHeader ::  String -> String
 htmlHeader title = unlines
   [ "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">"
   , "<html>"
   , "<head>"
-  ,"<!-- Generated by HsColour, http://www.cs.york.ac.uk/fp/darcs/hscolour/ -->"
+  ,"<!-- Generated by HsColour, http://code.haskell.org/~malcolm/hscolour/ -->"
   , "<title>"++title++"</title>"
   , "</head>"
   , "<body>"
   ]
+htmlClose ::  String
 htmlClose  = "\n</body>\n</html>"
